Attribute VB_Name = "DBCode"
Option Explicit

Private m_DBConnection As ADODB.Connection
' If the connection is open, close it.
Public Sub CloseDatabase()
    ' See if the connection object exists.
    If Not m_DBConnection Is Nothing Then
        ' See if the connection is open.
        If m_DBConnection.State = adStateOpen Then
            m_DBConnection.Close
        End If
    End If

    Set m_DBConnection = Nothing
End Sub
' Execute an action statement.
Public Sub ExecuteActionStatement(ByVal statement As String)
    m_DBConnection.Execute statement, , adCmdText
End Sub

' Execute a query that returns one column and return
' a collection of the values selected.
Public Function ExecuteOneColumnQuery(ByVal query As String) As Collection
Dim results As Collection
Dim rs As ADODB.Recordset

    Set results = New Collection

    ' Execute the query.
    Set rs = m_DBConnection.Execute(query, , adCmdText)

    ' Save the results in the collection.
    Do Until rs.EOF
        results.Add rs.Fields(0).Value
        rs.MoveNext
    Loop

    ' Close the Recordset.
    rs.Close

    ' Return the collection.
    Set ExecuteOneColumnQuery = results
End Function
' Execute a query that returns one row and return
' a collection of the values selected.
Public Function ExecuteOneRowQuery(ByVal query As String) As Collection
Dim results As Collection
Dim rs As ADODB.Recordset
Dim field_num As Integer

    Set results = New Collection

    ' Execute the query.
    Set rs = m_DBConnection.Execute(query, , adCmdText)

    ' Save the results in the collection.
    If Not rs.EOF Then
        For field_num = 0 To rs.Fields.Count - 1
            results.Add rs.Fields(field_num).Value, _
                        rs.Fields(field_num).Name
        Next field_num
    End If

    ' Close the Recordset.
    rs.Close

    ' Return the collection.
    Set ExecuteOneRowQuery = results
End Function
' Return a collection of the column names returned
' by the query.
Public Function GetQueryFields(ByVal query As String) As Collection
Dim results As Collection
Dim rs As ADODB.Recordset
Dim field_num As Integer

    Set results = New Collection

    ' Execute the query.
    Set rs = m_DBConnection.Execute(query, , adCmdText)

    ' Save the field names in the collection.
    For field_num = 0 To rs.Fields.Count - 1
        results.Add rs.Fields(field_num).Name
    Next field_num

    ' Close the Recordset.
    rs.Close

    ' Return the collection.
    Set GetQueryFields = results
End Function
' Fill an MSFlexGrid with the results of a query.
' frm is a form with the same font as the MSFlexGrid.
Public Sub InitializeFlexGridFromQuery(ByVal frm As Form, ByVal flx As MSFlexGrid, ByVal query As String)
Dim rs As ADODB.Recordset
Dim num_cols As Integer
Dim col_widths() As Single
Dim r As Integer
Dim c As Integer
Dim txt As String

    ' Hide the MSFlexGrid.
    flx.Visible = False
    flx.Refresh

    ' Execute the query.
    Set rs = m_DBConnection.Execute(query, , adCmdText)

    ' Make sure we have at least one column.
    num_cols = rs.Fields.Count
    If num_cols < 1 Then Exit Sub

    ' Give the MSFlexGrid enough columns.
    flx.Cols = num_cols
    flx.FixedCols = 0
    flx.FixedRows = 0

    ' Display the headers.
    flx.Rows = 1
    ReDim col_widths(0 To num_cols - 1)
    For c = 0 To num_cols - 1
        txt = rs.Fields(c).Name
        flx.TextMatrix(0, c) = txt
        flx.ColAlignment(c) = flexAlignLeftCenter
        col_widths(c) = frm.TextWidth("> " & txt)
    Next c

    ' Display the values.
    r = 1
    Do Until rs.EOF
        flx.Rows = r + 1
        For c = 0 To num_cols - 1
            txt = rs.Fields(c).Value & ""
            flx.TextMatrix(r, c) = txt
            If col_widths(c) < frm.TextWidth(txt) Then
                col_widths(c) = frm.TextWidth(txt)
            End If
        Next c

        r = r + 1
        rs.MoveNext
    Loop

    ' Close the Recordset.
    rs.Close

    ' Adjust the column widths.
    For c = 0 To num_cols - 1
        flx.ColWidth(c) = col_widths(c) + 120
    Next c

    ' Display the MSFlexGrid.
    flx.FixedCols = 0
    If flx.Rows > 1 Then flx.FixedRows = 1
    flx.Visible = True
End Sub
' Initialize a ComboBox using the values returned by
' a single-column query.
Public Sub InitializeComboFromQuery(ByVal cbo As ComboBox, ByVal query As String, Optional ByVal allow_blank As Boolean = False)
Dim results As Collection
Dim i As Integer

    ' Execute the query.
    Set results = ExecuteOneColumnQuery(query)

    ' If we should allow blanks, start with a blank.
    cbo.Clear
    If allow_blank Then cbo.AddItem ""

    ' Initialize the ComboBox.
    For i = 1 To results.Count
        cbo.AddItem results(i)
    Next i
End Sub
' Initialize a ComboBox using the field names returned
' by a query.
Public Sub InitializeComboFromQueryFields(ByVal cbo As ComboBox, ByVal query As String, Optional ByVal allow_blank As Boolean = False)
Dim results As Collection
Dim i As Integer

    ' Execute the query.
    Set results = GetQueryFields(query)

    ' If we should allow blanks, start with a blank.
    cbo.Clear
    If allow_blank Then cbo.AddItem ""

    ' Initialize the ComboBox.
    For i = 1 To results.Count
        cbo.AddItem results(i)
    Next i
End Sub

' Display the results of the query in
' the MSFlexGrid control.
Public Function ExecuteMultiColumnQuery(ByVal query As String) As Variant
Dim rs As ADODB.Recordset

    ' Execute the query.
    Set rs = m_DBConnection.Execute(query, , adCmdText)

    ' Return the results.
    ExecuteMultiColumnQuery = rs.GetRows()

    ' Close the Recordset.
    rs.Close
End Function
' Add the current path to the file name. Add a "\" if
' needed (App.Path does not end in one if the path
' is a root such as "C:").
Public Function MakeDBName(ByVal file_name As String) As String
    If Right$(App.Path, 1) = "\" Then
        MakeDBName = App.Path & file_name
    Else
        MakeDBName = App.Path & "\" & file_name
    End If
End Function

' Open the database.
Public Sub OpenDatabase(ByVal db_name As String)
    ' If the connection is open, close it.
    If Not m_DBConnection Is Nothing Then
        If m_DBConnection.State = adStateOpen Then
            m_DBConnection.Close
        End If
    End If

    ' Open the connection.
    Set m_DBConnection = New ADODB.Connection
    m_DBConnection.Open _
        "Provider=Microsoft.Jet.OLEDB.3.51;" & _
        "Persist Security Info=False;" & _
        "Data Source=" & db_name
End Sub
' Make the string safe to send into SQL by replacing
' the ' character with ''.
Public Function SQLSafe(ByVal txt As String) As String
    SQLSafe = Replace(txt, "'", "''")
End Function
